/***************************************************************************
 *   This file is part of the Inanna library.                              *
 *                                                                         *
 *   Copyright (C) 1997-2002 Marko Grnroos <magi@iki.fi>                  *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 *  This library is free software; you can redistribute it and/or          *
 *  modify it under the terms of the GNU Library General Public            *
 *  License as published by the Free Software Foundation; either           *
 *  version 2 of the License, or (at your option) any later version.       *
 *                                                                         *
 *  This library is distributed in the hope that it will be useful,        *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of         *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU      *
 *  Library General Public License for more details.                       *
 *                                                                         *
 *  You should have received a copy of the GNU Library General Public      *
 *  License along with this library; see the file COPYING.LIB.  If         *
 *  not, write to the Free Software Foundation, Inc., 59 Temple Place      *
 *  - Suite 330, Boston, MA 02111-1307, USA.                               *
 *                                                                         *
 ***************************************************************************/

#ifndef __RPROPTRAINER_H__
#define __RPROPTRAINER_H__

#include "inanna/trainer.h"
#include "inanna/backprop.h"

///////////////////////////////////////////////////////////////////////////////
//        ----  ----                -----           o                        //
//        |   ) |   )           --    |        ___      _    ___             //
//        |---  |---  |/\  __  |  )   |   |/\  ___| | |/ \  /   ) |/\        //
//        | \   |     |   /  \ |--    |   |   (   | | |   | |---  |          //
//        |  \  |     |   \__/ |      |   |    \__| | |   |  \__  |          //
///////////////////////////////////////////////////////////////////////////////

/** Resilient error backpropagation algorithm by Riedmiller.
 *
 *  Design Patterns: Template Method (various parts of the algorithm
 *  can be overloaded).
 **/
class RPropTrainer : public BackpropTrainer {
  public:
	virtual Array<DynParameter>*	parameters	() const;
	virtual void					init		(const StringMap& params);
	
  protected:

	/** Implementation for BackpropTrainer. Initializes training. */
	virtual void					initTrain		(ANNetwork& network) const;

	/** Implementation for BackpropTrainer. */
	virtual void					backpropagate	(ANNetwork& network, const PatternSet& set, int p) const;

	/** Updates weights after backpropagation phase. */
	virtual void					updateWeights	(ANNetwork& network) const;

  protected:
	/** Initial per-weight delta. */
	double	mDelta0;

	/** Maximum per-weight delta. */
	double	mDeltaMax;

	/** Per-weight deltas.
	 *
	 *  We store these here, because we don't want to alter the
	 *  network objects just because of the training algorithm.
	 **/
	mutable Vector	mDelta;

	/** Per-weight ...
	 *
	 *  We store these here, because we don't want to alter the
	 *  network objects just because of the training algorithm.
	 **/
	mutable Vector	mOldDeltaW;

	/** Per-weight summed error.
	 *
	 *  We store these here, because we don't want to alter the
	 *  network objects just because of the training algorithm.
	 **/
	mutable Vector	mGradient;
};

#endif
