/***************************************************************************
 *                                                                         *
 *   Copyright (C) 2001-2004 Marko Grnroos <magi@iki.fi>                  *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef __MAGIC_VMCOMPILER_H_
#define __MAGIC_VMCOMPILER_H_

#include <magic/mtextstream.h>
#include <vm.h>

/*******************************************************************************
 * Definitions
 ******************************************************************************/

/*******************************************************************************
 * Language token type
 ******************************************************************************/
enum vmtoken {
	VMT_EOS       = 0,  // End-of-stream reached
	VMT_INT		  = 1,  // Integer value
	VMT_LITERAL	  = 2,  // String literal
	VMT_LABEL     = 4,  // An address label, such as mylabel:
	VMT_VARREF    = 5,  // Variable reference, such as $myvar
	VMT_NAME      = 6,  // A statement, native function call, or label ref.
	VMT_ERROR     = 100
};

///////////////////////////////////////////////////////////////////////////////
//                                                                           //
//                   |   | |   | |                                           //
//                   |   | |\ /| |      ___       ___                        //
//                   |   | | V | |     /   ) \ / /   ) |/\                   //
//                    \ /  | | | |     |---   X  |---  |                     //
//                     V   |   | |____  \__  / \  \__  |                     //
//                                                                           //
///////////////////////////////////////////////////////////////////////////////

class VMLexer {
  public:
	VMLexer ();

	vmtoken			readToken	(TextIStream& in);
	int				getInt		() const {return mIntValue;}
	const String&	getString	() const {return mStringValue;}
	int				currentLine	() const {return mCurrentLine;}
	
  private:
	String	mStringValue;
	int		mIntValue;
	int		mCurrentLine;
};

///////////////////////////////////////////////////////////////////////////////
//                                                                           //
//              |   | |   |  ___                  o |                        //
//              |   | |\ /| /   \             --    |  ___                   //
//              |   | | V | |      __  |/|/| |  ) | | /   ) |/\              //
//               \ /  | | | |     /  \ | | | |--  | | |---  |                //
//                V   |   | \___/ \__/ | | | |    | |  \__  |                //
//                                                                           //
///////////////////////////////////////////////////////////////////////////////

/*******************************************************************************
 * A compiler for compiling source code to bytecode
 ******************************************************************************/
class VMCompiler {
  public:
				VMCompiler	();
	vmresult	compile		(TextIStream& in, String& program);

  private:
	struct CodeReference {
		CodeReference (int pos, const String& cref, int cline)
				: position (pos), reference (cref), lineno (cline) {}
		
		int		position;
		String	reference;
		int		lineno;
	};

	String		mProgram;
};

#endif
