/***************************************************************************
 *                                                                         *
 *   Copyright (C) 1998-2004 Marko Grnroos <magi@iki.fi>                  *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <vm.h>
#include <magic/mtextstream.h>

/*******************************************************************************
 * Native function library singleton
 ******************************************************************************/
VMNativeLib vmNativeLib;

//////////////////////////////////////////////////////////////////////////////
//                                                                          //
//       |   |           o                                                  //
//       |\  |  ___   |           ___      __         _    ___   ____       //
//       | \ |  ___| -+- | |   | /   )    /   |   | |/ \  |   \ (           //
//       |  \| (   |  |  |  \ /  |---     +-- |   | |   | |      \__        //
//       |   |  \__|   \ |   V    \__     |    \__! |   |  \__/ ____)       //
//                                        |                                 //
//////////////////////////////////////////////////////////////////////////////

vmfresult vmf_equal (VM& vm, VMThread& thread)
{
	thread.push (thread.pop() == thread.pop ());
	return 0;
}

vmfresult vmf_plus (VM& vm, VMThread& thread)
{
	thread.push (thread.pop() + thread.pop ());
	return 0;
}

vmfresult vmf_minus (VM& vm, VMThread& thread)
{
	thread.push (- (thread.pop() - thread.pop ()));
	return 0;
}

vmfresult vmf_prints (VM& vm, VMThread& thread)
{
	int addr = thread.pop ();
	String printable;
	for (int pos = addr; vm.memory()[pos] != 0; pos++)
		printable += (char) vm.memory()[pos];

	sout << printable;
	return 0;
}

vmfresult vmf_printi (VM& vm, VMThread& thread)
{
	sout << thread.pop ();
	return 0;
}

#include <sys/poll.h>

/*******************************************************************************
 * Reads integer values from stdin
 ******************************************************************************/
vmfresult vmf_readInt (VM& vm, VMThread& thread)
{
	/* Poll to check if there is input available. */
	struct pollfd stdin_pollfd;
	stdin_pollfd.fd     = 1; // Stdin
	stdin_pollfd.events = POLLIN | POLLPRI;

	int pollresult = poll (&stdin_pollfd, 1,
						   0); // Timeout

	if (pollresult > 0) {
		/* There is input available. Read it. */
		char buffer[81];
		scanf ("%80s", buffer);
		if (isdigit (buffer[0])) {
			int readval = atoi (buffer);
			thread.push (readval);
			return 0;
		} else
			serr << "Error: non-integer value read.\n";
	}

	return VMERR_KEEP_POSITION;
}

///////////////////////////////////////////////////////////////////////////////
//                                                                           //
//          |   | |   | |   |           o             |     o                //
//          |   | |\ /| |\  |  ___   |           ___  |       |              //
//          |   | | V | | \ |  ___| -+- | |   | /   ) |     | |---           //
//           \ /  | | | |  \| (   |  |  |  \ /  |---  |     | |   )          //
//            V   |   | |   |  \__|   \ |   V    \__  |____ | |__/           //
//                                                                           //
///////////////////////////////////////////////////////////////////////////////

/*******************************************************************************
 * Default constructor.
 *
 * Registers all native functions automatically.
 *******************************************************************************/
VMNativeLib::VMNativeLib ()
{
	registerFunc ("==", vmf_equal);
	registerFunc ("+", vmf_plus);
	registerFunc ("-", vmf_minus);
	registerFunc ("Prints", vmf_prints);
	registerFunc ("Printi", vmf_printi);
	registerFunc ("ReadInt", vmf_readInt);
}

void VMNativeLib::registerFunc (const String& name, vmnativefunc func)
{
	VMNativeFunc* pNewFunc = new VMNativeFunc;
	pNewFunc->name = name;
	pNewFunc->func = func;

	mNatives.add (pNewFunc);
}

int VMNativeLib::resolve (const String& name) const
{
	for (int i=0; i<mNatives.size(); i++)
		if (mNatives[i].name == name)
			return i; // Return function id
	return -1;
}
