/***************************************************************************
 *                                                                         *
 *   Copyright (C) 2004 Marko Grnroos <magi@iki.fi>                       *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <magic/mapplic.h>
#include <magic/mtextstream.h>
#include <vm.h>

/*******************************************************************************
 * Definitions
 ******************************************************************************/

/* Application exit values */
#define VMRUN_EXIT_INVALID_PARAMETERS  1
#define VMRUN_EXIT_LOAD_FAILED         2

/*******************************************************************************
 * Prints command-line usage message
 ******************************************************************************/
void usage ()
{
	sout << "Usage: mtvmrun <mtvm_binary>\n"
		 << "Executes a compiled program in virtual machine.\n";
}

/*******************************************************************************
 * Loads VM executable from file
 ******************************************************************************/
vmresult load (VM& vm, const String& objfile)
{
	// Open file
	File infile (objfile);
	if (!infile.open (IO_Readable)) {
		serr << "Error: Could not open file '" << objfile << "'.\n";
		return VMERR_LOAD_FAILED;
	}

	// Read file contents to a buffer string
	Ref<String> contents = infile.readAll ();
	if (contents.isNull() || contents->length () == 0) {
		serr << "Error: Empty file.\n";
		return VMERR_LOAD_FAILED;
	}

	// An assumption
	ASSERT (sizeof(vmword) == sizeof(char));
	
	// Copy the file to memory
	int memsize = contents->length();
	vm.memory().make (memsize);
	for (int i=0; i<memsize; i++)
		vm.memory()[i] = contents.object()[i];
	
	return 0;
}

///////////////////////////////////////////////////////////////////////////////
//                                                                           //
//                            |   |       o                                  //
//                            |\ /|  ___      _                              //
//                            | V |  ___| | |/ \                             //
//                            | | | (   | | |   |                            //
//                            |   |  \__| | |   |                            //
//                                                                           //
///////////////////////////////////////////////////////////////////////////////

Main ()
{
	// Read command-line parameters
	if (params().size () != 1) {
		usage ();
		exit (VMRUN_EXIT_INVALID_PARAMETERS);
	}
	String filename = params()[0];

	// Create virtual machine
	VM myVm;

	// Load binary file into virtual machine memory
	if (load (myVm, filename) != 0) {
		serr << "Could not load binary file.\n";
		exit (VMRUN_EXIT_LOAD_FAILED);
	}

	// Run the virtual machine
	myVm.run ();
}

