/***************************************************************************
 *   This file is part of the MagiC++ library.                             *
 *                                                                         *
 *   Copyright (C) 1998-2002 Marko Grnroos <magi@iki.fi>                  *
 *                                                                         *
 ***************************************************************************
 *                                                                         *
 *  This library is free software; you can redistribute it and/or          *
 *  modify it under the terms of the GNU Library General Public            *
 *  License as published by the Free Software Foundation; either           *
 *  version 2 of the License, or (at your option) any later version.       *
 *                                                                         *
 *  This library is distributed in the hope that it will be useful,        *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of         *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU      *
 *  Library General Public License for more details.                       *
 *                                                                         *
 *  You should have received a copy of the GNU Library General Public      *
 *  License along with this library; see the file COPYING.LIB.  If         *
 *  not, write to the Free Software Foundation, Inc., 59 Temple Place      *
 *  - Suite 330, Boston, MA 02111-1307, USA.                               *
 *                                                                         *
 ***************************************************************************/

#define private public
#include "magic/mobject.h"
#include "magic/mapplic.h"
#include "magic/mclass.h"
#include "magic/mtextstream.h"

#include <time.h>

BEGIN_NAMESPACE (MagiC);

impl_dynamic (Application, {Object});

END_NAMESPACE;

using namespace MagiC;

/*
  Does not work with EGCS
  
void classDump (const Object& obj) {
	const type_info& tir=typeid(obj);
	int nodetype = tir.__rtti_get_node_type ();
	if (nodetype != type_info::_RTTI_CLASS_TYPE) {
		serr.print ("Node type requested for a non-class object\n");
		return;
	}
	
	const __class_type_info& ci = (const __class_type_info&) tir;
	serr.printf ("class %s ", ci.name ());
	for (int i=0; i<ci.n_bases; i++) {
		const type_info& clbase = *ci.base_list[i];
		serr.printf (": %s ", clbase.name());
	}
	serr.print ("\n");
}
*/

//////////////////////////////////////////////////////////////////////////////
//                                                                          //
//               _             | o                 o                        //
//              / \   --   --  |    ___   ___   |           _               //
//             /   \ |  ) |  ) | | |   \  ___| -+- |  __  |/ \              //
//             |---| |--  |--  | | |     (   |  |  | /  \ |   |             //
//             |   | |    |    | |  \__/  \__|   \ | \__/ |   |             //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////

void myterminate () {
	printf ("%%ERR: Program terminated for unspecified reason\n"
			"%%ERR: This is possibly due to a bad exception\n"
		    "%%ERR: or some other internal error\n");
	exit (1);
}

void myunexpected () {
	printf ("%%ERR: Bad exception was thrown. Exiting...\n");
	exit (1);
}

// Defined in mobject.cc
extern int newtrace_disabled;

int main (int argc, char** argv) {
	set_terminate (myterminate);
	set_unexpected (myunexpected);

	randomize;

#ifndef DISABLE_ALL_MEMORY_DEBUGGING
	// Don't trace allocations before this point, since we DON'T want
	// to trace global objects, like class and module objects
	newtrace_disabled = false;
#endif
	
	if (appclass) {
		// Create instance of the (user-inherited) application class
		newComment ("Application instance");
		Application* app = static_cast <Application*> (
			&(ClassLib::getinstance (appclass->getname())));

		// If such was found...
		if (app) {
			app->setParams (argc, argv);

			// Run any one of it's main-methods
			try {
				app->main (app->params());
				if (app->getTried()==1)
					app->main ();
			} catch (assertion_failed e) {
				// Catch any uncaught exceptions
				fprintf (stderr, "%%EXCEPTION: Caught at main level\n %s\n",
						 (CONSTR) e.what());
				exit (1);
			} catch (exception e) {
				fprintf (stderr,
						 "%%exception: Caught at main level\n %s\n", e.what());
				exit (1);
			} catch (Exception e) {
				fprintf (stderr,
						 "%%Exception: Caught at main level\n %s\n", (CONSTR) e.what());
				exit (1);
			}
			delete app;
		} else {
			// No user-defined Application inheritor was found
			serr.printf ("%%ERR: Internal error in Application "
						 "class creation\n"
						 "%%ERR: An Application class was found "
						 "with name '%s',\n"
						 "%%ERR: but it was not possible to get an instance "
						 "for it\n",
						 (CONSTR) appclass->getname());
			// classDump (*appclass);
		}
	} else {
		fprintf (stderr, "%%ERR: No Application decendant found\n");
	}

#ifndef DISABLE_ALL_MEMORY_DEBUGGING
	// Check for memory leaks
	if (dbgnewmalloced>0) {
		fprintf (stderr, "%%ERR: Detected memory leaks!!!!\n");
		list_allocations ();
	}
#endif

	return 0;
}

BEGIN_NAMESPACE (MagiC);

/** Sets command-line parameter list and parameter map
 *  according to the command-line parameters.
 **/
void Application::setParams (int argc, char** argv)
{
	mArgc = argc;
	mArgv = argv;

	// Move the parameters to String array
	
	Array<String>	params;
	if (argc>1)
		params.resize (argc-1);
	
	for (int i=0; i<params.size(); i++)
		params [i] = argv[i+1];

	mAppPath = argv[0];

	initConfigPath ();

	// Parse any parameters of the format "-xxxx=yyyyy" into a parameter map
	StringMap paramMap;
	for (int p=0; p<params.size(); p++)
		if (params[p].length()>1 && params[p][0]=='-') {
			String prm2 = params[p].mid(1);
			Array<String> lr;
			prm2.split (lr, '=');
			if (lr.size() == 1) { // A normal flag - not handled here
				mParams.add (params[p]); // (Same as below)
			} else if (lr.size() > 2) { // Multiple =-signs are not allowed
				serr.printf ("Parameters must follow format: -param=value\n");
				exit (1);
			} else // Add to parameter map
				mParamMap.set (lr[0],lr[1]);
		} else
			mParams.add (params[p]);
}

void Application::initConfigPath ()
{
	// Initialize configuration file search path
	mConfigPath.add ("");

	String appdir  = "";
	String appname = "";
	int slashpos = mAppPath.findRev ("/");
	if (slashpos != -1) {
		appdir  = mAppPath.mid (0, slashpos);
		appname = mAppPath.mid (slashpos+1, -1);
	}

	// Add default configuration directory
	String configdir = appdir + "/../share/apps/" + appname + "/config/";
	mConfigPath.add (configdir);

	mDataDir = appdir + "/../share/apps/" + appname + "/data/";
}

/** Reads the application configuration file on top of the
 *  current parameter map.
 **/
void Application::readConfig (const String& filename)
{
	bool notfound = true;
	for (int i=0; i<mConfigPath.size(); i++) {
		String configfile = mConfigPath[i] + filename;
		try {
			// Try the read the configuration file
			StringMap paramMap = readStringMap (configfile);
			
			// Got it.
			// Add the command-line parameters on top of those
			paramMap += mParamMap;
			mParamMap = paramMap;

			notfound = false;
			break;
		} catch (Exception e) {
		}
	}

	if (notfound)
		throw file_not_found (strformat ("Configuration file '%s' not found.", (CONSTR) filename));

	// Parameters are from now on REQUIRED
	mParamMap.failByThrow ();
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////

END_NAMESPACE;
