/*
  Strategian Tien perusta kouluni nkkulmasta selitetn Maan
  kirjassa. On vaikeaa ymmrt todellinen Tie vain miekkailun
  avulla. Tied pienimmt asiat ja suurimmat asiat, matalimmat asiat
  ja syvimmt asiat. Kuin se olisi suora tie kirjoitettuna maahan,
  ensimmist kirjaa kutsutaan Maan kirjaksi.
  */

#ifndef __GAENVRNMNT_H__
#define __GAENVRNMNT_H__

#include <magic/mobject.h>

using namespace MagiC;

/** Reference type. EXPERIMENTAL.
 **/
/*
template <class TYPE>
class Ref : public Object {
  protected:
	TYPE*	obj;
  public:
				Ref						(TYPE* o) {obj = o;}
				Ref						(Ref<TYPE>& o) {obj = o.obj; o.obj=NULL;}
				~Ref					() {delete obj;}
				operator =				(TYPE* n) {delete obj; obj=n;}
	//				operator const TYPE&	() const {return *obj;}
				operator TYPE&			() {return *obj;}
	TYPE*		operator ->				() {return obj;}
	TYPE*		getptr					() {return obj;}
	//	TYPE*		operator &				() {return obj;}
	//	const TYPE*	operator &				() const {return obj;}
};
*/

// Externals
class Genome;
class Individual;

/** The abstract base class for environments ("objective functions")
 *  where the fitness of Individuals is measured.
 *
 *  The inheritors of this class are typically used as follows (assume
 *  that the parameter map has been defined and parameters to it
 *  inserted):
 *
 *	MyEnvironment myEnvir (paramMap, someOtherParameters);
 *
 *  SimplePopulation pop (envir, paramMap);
 *
 *	pop.evolve (100);
 *  
 *  Note: the class is actually not abstract because the RTTI system
 *  doesn't allow that.
 **/
class EAEnvironment : public Object {
	decl_dynamic (EAEnvironment);
  public:

					EAEnvironment	();

	/** Sets the number of evaluations to be done for each individual
	 *  to determine it's fitness. This is useful only if the fitness
	 *  measurement is noisy.
	 *
	 *  This variable should be set before starting the evolution, of
	 *  course.
	 **/
	void			setevals		(int n) {mNEvals = n;}

	/** Returns the number of evaluations to be done for each
	 *  individual to determine it's fitness.
	 **/
	int				evals			() const {return mNEvals;}

	/** Returns the total number of evaluations performed in all
	 *  generations so far. This includes all multiple evaluations of
	 *  all individuals.
	 **/
	int				total_evals		() const {return mTotEvals;}

	/** Orders to add gaussian noise to the evaluated fitness. This is
	 *  useful for testing the robustness of the EA for noisy
	 *  evaluation.
	 **/
	void			addnoise		(double stddev) {mNoise = stddev;}

	double			evaluate		(const Individual& indiv);

	/** Sets evolution log directory for cycle reports and
	 *  miscellaneous log files.
	 **/
	void			logDir			(const String& dir) {mLogDir = dir;}

	/** Prints out a generation report to the given brief log stream
	 *  and more verbose output stream.
	 **/
	void			cycleReport		(OStream& log, OStream& out) {mCycles++; cycle_report(log,out);}
	
	// Virtual methods
	
	/** Adds problem-specific features to a genome.
	 **/
	virtual void	addFeaturesTo	(Genome& genome) const {;}

	/** Initializes a generation. Not necessary in all models.
	 **/
	virtual void	init_cycle		() {mBestFitness = 99999999.0; mpBest = NULL;}

	/** Implementation for @ref Object. */
	virtual void		check			() const;
	
	// Lowest non-subjective fitness; unlike the fitness returned by
	// the evaluate-method, this does NOT include the artificial noise
	// factor.
	double			bestfitn;
	
  protected:
	/** Evaluates the fitness of the given individual in the
	 *  environment. MUST OVERLOAD!
	 *
	 *  @exception must_overload
	 **/
	virtual double	evaluateg		(const Individual& ind) {MUST_OVERLOAD; return 0.0;}

	/** Prints some statistics or something at the end of the evaluation cycle.
	 **/
	virtual void	cycle_report	(OStream& log, OStream& out) {;}

	/** Number of evaluations to be done for each individual, default=1.
	 **/
	int				mNEvals;

	/** Total number of evaluations done so far.
	 **/
	int				mTotEvals;

	/** Basicly a generation counter. It would be nicer if this was not
	 *  stored here (redundacy, see).
	 **/
	int				mCycles;

	/** Amount of artificial noise to be added to evaluated
	 *  fitness. This is useful for testing the effects of noise on
	 *  evolution. Requires deterministic test functions to be useful.
	 **/
	double			mNoise;

	/** The best individual in this cycle.
	 **/
	Individual*		mpBest;

	/** (Subjective) fitness of the best individual. "Subjective"
	 *  means here that the artificial noise is included in this value.
	 **/
	double			mBestFitness;

	/** Directory for gathering evolution logs.
	 **/
	String			mLogDir;
};

#endif
